import { ApiItem } from '@microsoft/api-extractor-model';
import { ApiModel } from '@microsoft/api-extractor-model';
import { ApiPackage } from '@microsoft/api-extractor-model';
import { DocBlock } from '@microsoft/tsdoc';
import { DocSection } from '@microsoft/tsdoc';

/**
 * Configuration
 *
 * @public
 */
export declare interface Config {
    /**
     * markdown link reference {@link ReferenceResolver | resolver}
     */
    linkReferencer?: ReferenceResolver;
    /**
     * markdown docs {@link MarkdownProcessor | processor}
     */
    processor: MarkdownProcessor;
}

/**
 * Content Builder
 *
 * @public
 */
export declare interface ContentBuilder {
    /**
     * Indent level
     * @defaultValue 0
     */
    readonly indentLevel: number;
    /**
     * Content
     */
    readonly content: string;
    /**
     * Add content
     * @param content - additional content
     */
    push(content: string): void;
    /**
     * Add line break
     */
    newline(): void;
    /**
     * Add content with line break
     * @param content - additional content
     */
    pushline(content: string): void;
    /**
     * Indent content
     * @param withNewLine - whether indent to be added without new line
     */
    indent(withoutNewLine?: boolean): void;
    /**
     * DeIndent content
     * @param withoutNewLine - whether deindent to be added without new line
     */
    deindent(withoutNewLine?: boolean): void;
}

/**
 * Content Builder options
 *
 * @remarks
 * options that creating a {@link ContentBuilder}
 *
 * @public
 */
export declare interface ContentBuilderOptions {
    /**
     * Intdent level of Builder
     */
    indentLevel?: number;
}

/**
 * Create a Content Builder
 *
 * @param options - Content Builder options
 *
 * @returns A {@link ContentBuilder} instance
 *
 * @public
 */
export declare function createContentBuilder(options?: ContentBuilderOptions): ContentBuilder;

/**
 * Default Configration
 *
 * @remarks
 * Default config for the CLI
 *
 * @public
 */
export declare const DefaultConfig: Config;

/**
 * Escape text
 *
 * @param text - the target text
 *
 * @returns escaped text
 *
 * @public
 */
export declare function escapeText(text: string): string;

/**
 * Find custom tags
 *
 * @remarks
 * About custom tags, See the {@link https://github.com/microsoft/tsdoc/issues/21 | this issue}
 *
 * @param customBlocks - target cusotm blocks
 * @param tag - finding target tag
 *
 * @returns found custom blocks
 *
 * @public
 */
export declare function findCustomTags(customBlocks: readonly DocBlock[], tag: string): DocBlock[];

/**
 * Generate API docs
 *
 * @param input - input paths
 * @param output - output api docs full path
 * @param options - optiosn for generate, see the {@link GenerateOptions}
 *
 * @public
 */
export declare function generate(input: string[], output: string, options: GenerateOptions): Promise<void>;

/**
 * Generate Options for Generate API
 *
 * @public
 */
export declare interface GenerateOptions {
    /**
     * generate style
     *
     * @remarks
     * see the {@link GenerateStyle}
     */
    style: GenerateStyle;
    /**
     * configration
     *
     * @remarks
     * see the {@link Config}
     */
    config: Config;
    /**
     * TSDoc configration path
     *
     * @remarks
     * Optional, see the {@link https://github.com/microsoft/tsdoc/tree/master/tsdoc-config | here}
     */
    tsdocConfigPath?: string;
    /**
     * generate done callback
     *
     * @remarks
     * The callback that will be called when the generate process is finished.
     */
    done?: (pkgname: string, filename: string) => void;
    /**
     * TSDoc configration error callback
     *
     * @remarks
     * The callback occurs if you have an error in configration when `--tsdoc-config` is specified
     */
    errorOnTSDocConfig?: (error: string) => void;
}

/**
 * The generate style
 *
 * @remarks
 * The value of this constants is the same as that taken from `--genereate-style` option.
 *
 * @public
 */
export declare const enum GenerateStyle {
    /**
     * Prefix the output api docs file name with package name
     */
    Prefix = "prefix",
    /**
     * Output api docs files for each package
     */
    Directory = "directory"
}

/**
 * Get DocSection content
 *
 * @param model - a {@link https://rushstack.io/pages/api/api-extractor-model.apimodel/ | model}
 * @param pkg - a {@link https://rushstack.io/pages/api/api-extractor-model.apipackage/ | package}
 * @param content - a {@link https://github.com/microsoft/tsdoc/blob/master/tsdoc/src/nodes/DocSection.ts | content}
 * @param contextItem - a context {@link https://rushstack.io/pages/api/api-extractor-model.apiitem/ | item}
 * @param style - generate style, See the {@link GenerateStyle}
 * @param resolver - {@link ReferenceResolver | resolver} to resolve markdown content references
 *
 * @returns doc section markdown content
 *
 * @public
 */
export declare function getDocSectionContent(model: ApiModel, pkg: ApiPackage, content: DocSection, contextItem: ApiItem, style: GenerateStyle, resolver: ReferenceResolver, customTags: string[]): string;

/**
 * Get safe path from display name of {@link https://rushstack.io/pages/api/api-extractor-model.apiitem/ | ApiItem}
 *
 * @param name - the target displayname
 *
 * @returns safe path
 *
 * @public
 */
export declare function getSafePathFromDisplayName(name: string): string;

/**
 * Markdown content
 *
 * @public
 */
export declare interface MarkdownContent {
    /**
     * markdown filename
     */
    filename: string;
    /**
     * mkarkdown content
     */
    body: string;
}

/**
 * Markdown docs processor
 *
 * @param model - a {@link https://rushstack.io/pages/api/api-extractor-model.apimodel/ | model}
 * @param pkg - a {@link https://rushstack.io/pages/api/api-extractor-model.apipackage/ | package}
 * @param style - generate style. see the {@link GenerateStyle}
 * @param resolver - the markdown reference {@link ReferenceResolver | resolver}. if you're specfified at {@link Config}, it's passed, else it's not specified passed internal refenrece resolver.
 * @param customTags - TSDoc custom tags. This parameter is set to an array of custom tag names defined in `--tsdoc-config`.
 *
 * @returns markdown content
 *
 * @public
 */
export declare type MarkdownProcessor = (model: ApiModel, pkg: ApiPackage, style: GenerateStyle, resolver: ReferenceResolver, customTags?: string[]) => string | MarkdownContent[];

/**
 * Process of API doc model
 *
 * @remarks
 * Generate the markdown contents the bellow:
 *
 * ```
 * - Function
 * - Enum
 * - Interface
 * - Class
 * - Variable
 * - TypeAlias
 * ```
 *
 * @param model - a {@link https://rushstack.io/pages/api/api-extractor-model.apimodel/ | model}
 * @param pkg - a {@link https://rushstack.io/pages/api/api-extractor-model.apipackage/ | package}
 * @param style - generate style, See the {@link GenerateStyle}
 * @param resolver - {@link ReferenceResolver | resolver} to resolve markdown content references
 * @param customTags - TSDoc custom tags. This parameter is set to an array of custom tag names defined in `--tsdoc-config`.
 *
 * @returns markdown content strign or Array of {@link MarkdownContent}
 *
 * @public
 */
export declare function multiProcessor(model: ApiModel, pkg: ApiPackage, style: GenerateStyle, resolver: ReferenceResolver, customTags?: string[]): string | MarkdownContent[];

/**
 * Resolve the markdown content reference
 *
 * @remarks
 * This reference resolver is used by the {@link multiProcessor | processor} to generate API docs references for separate pieces of markdown content.
 *
 * @param style - generate style, See the {@link GenerateStyle}
 * @param item - a {@link https://rushstack.io/pages/api/api-extractor-model.apiitem/ | item}
 * @param model - a {@link https://rushstack.io/pages/api/api-extractor-model.apimodel/ | model}
 * @param pkg - a {@link https://rushstack.io/pages/api/api-extractor-model.apipackage/ | package}
 *
 * @returns resolved the reference string
 *
 * @public
 */
export declare function multiResolver(style: GenerateStyle, item: ApiItem, model: ApiModel, pkg: ApiPackage): string;

/**
 * Markdown reference resolver
 *
 * @param style - generate style, See the {@link GenerateStyle}
 * @param item - a {@link https://rushstack.io/pages/api/api-extractor-model.apiitem/ | item}
 * @param model - a {@link https://rushstack.io/pages/api/api-extractor-model.apimodel/ | model}
 * @param pkg - a {@link https://rushstack.io/pages/api/api-extractor-model.apipackage/ | package}
 *
 * @returns resolved reference path
 *
 * @public
 */
export declare type ReferenceResolver = (style: GenerateStyle, item: ApiItem, model: ApiModel, pkg: ApiPackage) => string;

/**
 * Process of API doc model
 *
 * @remarks
 * Generate the markdown contents that have TOC.
 * About API doc model, see the {@link https://api-extractor.com/pages/overview/demo_docs/ | doc model structure}, and {@link https://github.com/microsoft/rushstack/tree/master/apps/api-extractor-model | doc model API}.
 * In about generate api docs, see the {@link https://github.com/kazupon/api-docs-gen/blob/master/api-docs-gen-api.md | api-docs-gen API References}
 *
 * @param model - a {@link https://rushstack.io/pages/api/api-extractor-model.apimodel/ | model}
 * @param pkg - a {@link https://rushstack.io/pages/api/api-extractor-model.apipackage/ | package}
 * @param style - generate style, See the {@link GenerateStyle}
 * @param resolver - {@link ReferenceResolver | resolver} to resolve markdown content references
 * @param customTags - TSDoc custom tags. This parameter is set to an array of custom tag names defined in `--tsdoc-config`.
 *
 * @returns markdown string content that have TOC
 *
 * @public
 */
export declare function tocProcessor(model: ApiModel, pkg: ApiPackage, style: GenerateStyle, resolver: ReferenceResolver, customTags?: string[]): string | MarkdownContent[];

/**
 * Resolve the markdown content reference
 *
 * @remarks
 * This reference resolver is used by the {@link tocProcessor | processor} to generate a reference specifically for API docs of markdown content with TOC.
 *
 * @param style - generate style, See the {@link GenerateStyle}
 * @param item - a {@link https://rushstack.io/pages/api/api-extractor-model.apiitem/ | item}
 * @param model - a {@link https://rushstack.io/pages/api/api-extractor-model.apimodel/ | model}
 * @param pkg - a {@link https://rushstack.io/pages/api/api-extractor-model.apipackage/ | package}
 *
 * @returns resolved the reference string
 *
 * @public
 */
export declare function tocResolver(style: GenerateStyle, item: ApiItem, model: ApiModel, pkg: ApiPackage): string;

export { }
