"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.process = void 0;
const builder_1 = require("../builder");
const utils_1 = require("./utils");
/**
 * Process of API doc model
 *
 * @remarks
 * Generate the markdown contents the bellow:
 *
 * ```
 * - Function
 * - Enum
 * - Interface
 * - Class
 * - Variable
 * - TypeAlias
 * ```
 *
 * @param model - a {@link https://rushstack.io/pages/api/api-extractor-model.apimodel/ | model}
 * @param pkg - a {@link https://rushstack.io/pages/api/api-extractor-model.apipackage/ | package}
 * @param style - generate style, See the {@link GenerateStyle}
 * @param resolver - {@link ReferenceResolver | resolver} to resolve markdown content references
 * @param customTags - TSDoc custom tags. This parameter is set to an array of custom tag names defined in `--tsdoc-config`.
 *
 * @returns markdown content strign or Array of {@link MarkdownContent}
 *
 * @public
 */
function process(model, pkg, style, resolver, customTags) {
    // build
    function build(pkg) {
        const builders = new Map();
        const items = pkg.members[0] ? pkg.members[0].members : [];
        buildMarkdown(builders, items);
        return builders;
    }
    // build markdown
    function buildMarkdown(builders, items) {
        for (const item of items) {
            const { kind } = item;
            let builder = builders.get(kind);
            if (!builder) {
                builder = builder_1.createContentBuilder();
                builder.pushline(`# ${kind}`);
                builder.newline();
                builders.set(kind, builder);
            }
            switch (kind) {
                case "Function" /* Function */:
                    utils_1.buildFunctionContent(style, model, pkg, resolver, builder, item, customTags || []);
                    break;
                case "Enum" /* Enum */:
                    utils_1.buildEnumContent(style, model, pkg, resolver, builder, item, customTags || []);
                    break;
                case "Interface" /* Interface */:
                    utils_1.buildInterfaceContent(style, model, pkg, resolver, builder, item, customTags || []);
                    break;
                case "Class" /* Class */:
                    utils_1.buildClassContent(style, model, pkg, resolver, builder, item, customTags || []);
                    break;
                case "TypeAlias" /* TypeAlias */:
                    utils_1.buildTypeAliasContent(style, model, pkg, resolver, builder, item, customTags || []);
                    break;
                case "Variable" /* Variable */:
                    utils_1.buildVariableContent(style, model, pkg, resolver, builder, item, customTags || []);
                    break;
                default:
                    break;
            }
        }
    }
    function generate(builders) {
        const contents = [];
        for (const [kind, builder] of builders) {
            const content = builder.content;
            contents.push({
                filename: `${kind.toLowerCase()}.md`,
                body: content
            });
        }
        return contents;
    }
    // build markdown content!
    const builders = build(pkg);
    // generate markdown content!
    return generate(builders);
}
exports.process = process;
